@@ -27,7 +27,7 @@
  */
 
 char acct_sccsid[] =
-"@(#)acct.c	2.1  Copyright 1994 Livingston Enterprises Inc\n"
+"@(#)acct.c	2.5  Copyright 1994 Livingston Enterprises Inc\n"
 "		     Copyright 1997 Cistron Internet Services B.V.";
 
 
@@ -61,10 +61,14 @@ static char trans[64] =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 #define ENC(c) trans[c]
 
+static char porttypes[] = "ASITX";
+
+#define LOCK_LEN sizeof(struct radutmp)
+
 /*
  *	UUencode 4 bits base64. We use this to turn a 4 byte field
  *	(an IP adres) into 6 bytes of ASCII. This is used for the
- *	wtmp file if we didn't find a short name in the clients file.
+ *	wtmp file if we didn't find a short name in the naslist file.
  */
 char *uue(void *in)
 {
@@ -96,29 +100,28 @@ char *uue(void *in)
  */
 static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 {
-	UINT4		ip;
 	char		buf[32];
-	CLIENT		*cl;
+	NAS		*cl;
 	char		*s;
 
 	/*
 	 *	Fill out the UTMP struct for the radwtmp file.
 	 *	(this one must be "last" - compatible).
 	 */
-	ip = ut->framed_address;
 #ifdef __linux__
 	/*
 	 *	Linux has a field for the client address.
 	 */
-	wt->ut_addr = ip;
+	wt->ut_addr = ut->framed_address;
 #endif
 	/*
 	 *	We use the tty field to store the terminal servers' port
 	 *	and address so that the tty field is unique.
 	 */
-	if ((cl = client_find(ntohl(ut->nas_address))) != NULL)
+	s = "";
+	if ((cl = nas_find(ntohl(ut->nas_address))) != NULL)
 		s = cl->shortname;
-	if (s == NULL || s[0] == 0) s = uue(&ip);
+	if (s == NULL || s[0] == 0) s = uue(&(ut->nas_address));
 #if UT_LINESIZE > 9
 	sprintf(buf, "%03d:%s", ut->nas_port, s);
 #else
@@ -177,8 +180,8 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t)
 		/*
 		 *	Lock the utmp file, prefer lockf() over flock().
 		 */
-#ifdef F_WRLCK
-		(void)lockf(fd, 0, F_WRLCK);
+#if defined(F_LOCK) && !defined(BSD)
+		(void)lockf(fd, F_LOCK, LOCK_LEN);
 #else
 		(void)flock(fd, LOCK_EX);
 #endif
@@ -195,7 +198,10 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t)
 			/*
 			 *	Match. Zap it.
 			 */
-			lseek(fd, -sizeof(u), SEEK_CUR);
+			if (lseek(fd, -sizeof(u), SEEK_CUR) < 0) {
+				log(L_ERR, "Accounting: radzap: negative lseek!\n");
+				lseek(fd, 0, SEEK_SET);
+			}
 			u.type = P_IDLE;
 			u.time = t;
 			write(fd, &u, sizeof(u));
@@ -208,11 +214,6 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t)
 				fwrite(&wt, sizeof(wt), 1, fp);
 			}
 		}
-#ifdef F_WRLCK
-		(void)lockf(fd, 0, F_UNLCK);
-#else
-		(void)flock(fd, LOCK_UN);
-#endif
 		close(fd);
 	}
 	if (fp) fclose(fp);
@@ -255,12 +256,26 @@ int		dowtmp;
 
 	if (!rb_record &&
 	    (vp = pairfind(authreq->request, PW_USER_NAME)) == NULL) {
+		int check1 = 0;
+		int check2 = 0;
+
 		/*
-		 *	Looks like a reboot record (?). Buggy ComOS revisions
-		 *	send the wrong type of records...
+		 *	ComOS (up to and including 3.5.1b20) does not send
+		 *	standard PW_STATUS_ACCOUNTING_XXX messages.
+		 *
+		 *	Check for:  o no Acct-Session-Time, or time of 0
+		 *		    o Acct-Session-Id of "00000000".
+		 *
+		 *	We could also check for NAS-Port, that attribute
+		 *	should NOT be present (but we don't right now).
 	 	 */
 		if ((vp = pairfind(authreq->request, PW_ACCT_SESSION_TIME))
-			!= NULL && vp->lvalue != 0) {
+		     == NULL || vp->lvalue == 0)
+			check1 = 1;
+		if ((vp = pairfind(authreq->request, PW_ACCT_SESSION_ID))
+		     != NULL && memcmp(vp->strvalue, "00000000", 8) == 0)
+			check2 = 1;
+		if (check1 == 0 || check2 == 0) {
 			log(L_ERR, "Accounting: no username in record");
 			return -1;
 		}
@@ -272,15 +287,29 @@ int		dowtmp;
 		rb_record = 1;
 	}
 
+#ifdef NT_DOMAIN_HACK
+        if (!rb_record && vp) {
+		char buffer[128];
+		char *ptr;
+		if ((ptr = strchr(vp->strvalue, '\\')) != NULL) {
+			strncpy(buffer, ptr + 1, sizeof(buffer));
+			buffer[sizeof(buffer) - 1] = 0;
+			strcpy(vp->strvalue, buffer);
+		}
+	}
+#endif
 
 	/*
 	 *	Add any specific attributes for this username.
 	 */
-	if (!rb_record) hints_setup(authreq->request);
-
+	if (!rb_record) {
+		hints_setup(authreq->request);
+		presuf_setup(authreq->request);
+	}
 	time(&t);
 	memset(&ut, 0, sizeof(ut));
 	memset(&wt, 0, sizeof(wt));
+	ut.porttype = 'A';
 
 	/*
 	 *	First, find the interesting attributes.
@@ -313,6 +342,10 @@ int		dowtmp;
 				memcpy(ut.session_id, vp->strvalue,
 					sizeof(ut.session_id));
 				break;
+			case PW_NAS_PORT_TYPE:
+				if (vp->lvalue >= 0 && vp->lvalue <= 4)
+					ut.porttype = porttypes[vp->lvalue];
+				break;
 		}
 	}
 
@@ -339,13 +372,13 @@ int		dowtmp;
 	 */
 	if (status == PW_STATUS_ACCOUNTING_ON && nas_address) {
 		log(L_INFO, "NAS %s restarted (Accounting-On packet seen)",
-			client_name(nas_address));
+			nas_name(nas_address));
 		radzap(nas_address, -1, NULL, ut.time);
 		return 0;
 	}
 	if (status == PW_STATUS_ACCOUNTING_OFF && nas_address) {
 		log(L_INFO, "NAS %s rebooted (Accounting-Off packet seen)",
-			client_name(nas_address));
+			nas_name(nas_address));
 		radzap(nas_address, -1, NULL, ut.time);
 		return 0;
 	}
@@ -355,7 +388,7 @@ int		dowtmp;
 	 */
 	if (status != PW_STATUS_START && status != PW_STATUS_STOP) {
 		log(L_ERR, "NAS %s port %d unknown packet type %d)",
-			client_name(nas_address), ut.nas_port, status);
+			nas_name(nas_address), ut.nas_port, status);
 		return 0;
 	}
 
@@ -368,8 +401,8 @@ int		dowtmp;
 		/*
 		 *	Lock the utmp file, prefer lockf() over flock().
 		 */
-#ifdef F_WRLCK
-		(void)lockf(fd, 0, F_WRLCK);
+#if defined(F_LOCK) && !defined(BSD)
+		(void)lockf(fd, F_LOCK, LOCK_LEN);
 #else
 		(void)flock(fd, LOCK_EX);
 #endif
@@ -384,9 +417,15 @@ int		dowtmp;
 			if (status == PW_STATUS_STOP &&
 			    strncmp(ut.session_id, u.session_id, 
 			     sizeof(u.session_id)) != 0) {
-				log(L_ERR,
+				/*
+				 *	Don't complain if this is not a
+				 *	login record (some clients can
+				 *	send _only_ logout records).
+				 */
+				if (u.type == P_LOGIN)
+					log(L_ERR,
 		"Accounting: logout: entry for NAS %s port %d has wrong ID",
-				client_name(nas_address), u.nas_port);
+					nas_name(nas_address), u.nas_port);
 				r = -1;
 				break;
 			}
@@ -396,11 +435,14 @@ int		dowtmp;
 			    u.time >= ut.time) {
 				log(L_ERR,
 		"Accounting: login: entry for NAS %s port %d out of order",
-				client_name(nas_address), u.nas_port);
+				nas_name(nas_address), u.nas_port);
 				r = -1;
 				break;
 			}
-			lseek(fd, -sizeof(u), SEEK_CUR);
+			if (lseek(fd, -sizeof(u), SEEK_CUR) < 0) {
+				log(L_ERR, "Accounting: negative lseek!\n");
+				lseek(fd, 0, SEEK_SET);
+			}
 			r = 1;
 			break;
 		}
@@ -418,15 +460,10 @@ int		dowtmp;
 			} else if (r == 0) {
 				log(L_ERR,
 		"Accounting: logout: login entry for NAS %s port %d not found",
-				client_name(nas_address), ut.nas_port);
+				nas_name(nas_address), ut.nas_port);
 				r = -1;
 			}
 		}
-#ifdef F_WRLCK
-		(void)lockf(fd, 0, F_UNLCK);
-#else
-		(void)flock(fd, LOCK_UN);
-#endif
 		close(fd);
 	} else {
 		log(L_ERR, "Accounting: %s: %s", RADUTMP, strerror(errno));
@@ -448,17 +485,15 @@ int		dowtmp;
 
 
 
-int rad_accounting_orig(authreq, activefd)
-AUTH_REQ	*authreq;
-int		activefd;
+int rad_accounting_orig(AUTH_REQ *authreq, int activefd, int authtype)
 {
 	FILE		*outfd;
-	char		clientname[128];
+	char		nasname[128];
 	char		buffer[512];
 	char		*s;
 	VALUE_PAIR	*pair;
 	UINT4		nas;
-	CLIENT		*cl;
+	NAS		*cl;
 	long		curtime;
 	int		ret = 0;
 	struct stat	st;
@@ -469,10 +504,11 @@ int		activefd;
 	 */
 	if (stat(radacct_dir, &st) < 0)
 		return 0;
+	curtime = time(0);
 
 	/*
 	 *	Find out the name of this terminal server. We try
-	 *	to find the PW_NAS_IP_ADDRESS in the clients file.
+	 *	to find the PW_NAS_IP_ADDRESS in the naslist file.
 	 *	If that fails, we look for the originating address.
 	 *	Only if that fails we resort to a name lookup.
 	 */
@@ -480,39 +516,38 @@ int		activefd;
 	nas = authreq->ipaddr;
 	if ((pair = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL) {
 		nas = pair->lvalue;
-		if ((cl = client_find(nas)) != NULL) {
+		if ((cl = nas_find(nas)) != NULL) {
 			if (cl->shortname[0])
-				strcpy(clientname, cl->shortname);
+				strcpy(nasname, cl->shortname);
 			else
-				strcpy(clientname, cl->longname);
+				strcpy(nasname, cl->longname);
 		}
 	}
 	if (cl == NULL) {
 		s = ip_hostname(nas);
 		if (strlen(s) > 127 || strchr(s, '/'))
 			return -1;
-		strcpy(clientname, s);
+		strcpy(nasname, s);
 	}
 
 	/*
-	 *	Create a directory for this client.
+	 *	Create a directory for this nas.
 	 */
-	sprintf(buffer, "%s/%s", radacct_dir, clientname);
+	sprintf(buffer, "%s/%s", radacct_dir, nasname);
 	(void) mkdir(buffer, 0755);
 
 	/*
 	 *	Write Detail file.
 	 */
-	sprintf(buffer, "%s/%s/detail", radacct_dir, clientname);
+	sprintf(buffer, "%s/%s/detail", radacct_dir, nasname);
 	if((outfd = fopen(buffer, "a")) == (FILE *)NULL) {
 		log(L_ERR,
 			"Acct: Couldn't open file %s/%s/detail",
-				radacct_dir, clientname);
+				radacct_dir, nasname);
 		ret = -1;
 	} else {
 
 		/* Post a timestamp */
-		curtime = time(0);
 		fputs(ctime(&curtime), outfd);
 
 		/* Write each attribute/value to the log file */
@@ -521,7 +556,25 @@ int		activefd;
 			fputs("\t", outfd);
 			fprint_attr_val(outfd, pair);
 			fputs("\n", outfd);
-		pair = pair->next;
+			pair = pair->next;
+		}
+
+		/*
+		 *	Add non-protocol attibutes.
+		 */
+		fprintf(outfd, "\tTimestamp = %ld\n", curtime);
+		switch(authtype) {
+		    case 0:
+			fputs("\tRequest-Authenticator = Verified\n", outfd);
+			break;
+		    case 1:
+			fputs("\tRequest-Authenticator = None\n", outfd);
+			break;
+		    case 2:
+			fputs("\tRequest-Authenticator = Unverified\n", outfd);
+			break;
+		    default:
+			break;
 		}
 		fputs("\n", outfd);
 		fclose(outfd);
@@ -625,44 +678,120 @@ static int send_acct_reply(AUTH_REQ *authreq, VALUE_PAIR *reply,
 	return 0;
 }
 
-
-
 /*
- *	rad_accounting: call both the old and new style accounting functions.
- *	
- *	FIXME: we should verify the MD5 sum of the accounting request!
+ *	Validates the requesting client NAS.  Calculates the
+ *	signature based on the clients private key.
  */
-int rad_accounting(AUTH_REQ *authreq, int activefd)
+static int calc_acctdigest(u_char *digest, AUTH_REQ *authreq,
+			char *recvbuf, int len)
 {
-	int reply = 0;
-
-	if (rad_accounting_orig(authreq, activefd) == 0)
-		reply = 1;
-	if (rad_accounting_new(authreq, activefd, 1) == 0)
-		reply = 1;
-
-	if (!reply) return -1;
+	int	secretlen;
+	CLIENT	*cl;
+	char zero[AUTH_VECTOR_LEN];
 
 	/*
-	 *	Now send back an ACK to the NAS.
+	 *	See if we know this client.
 	 */
-	send_acct_reply(authreq, (VALUE_PAIR *)NULL, 
-			(char *)NULL,activefd);
-	pairfree(authreq->request);
-	memset(authreq, 0, sizeof(AUTH_REQ));
-	free(authreq);
+	if ((cl = client_find(authreq->ipaddr)) == NULL) {
+		log(L_ERR, "request from unknown client: %s",
+			client_name(authreq->ipaddr));
+		return -1;
+	}
 
-	return 0;
+	/*
+	 *	Copy secret into authreq->secret so that we can
+	 *	use it with send_acct_reply()
+	 */
+	secretlen = strlen(cl->secret);
+	strcpy(authreq->secret, cl->secret);
+
+	/*
+	 *	Older clients have the authentication vector set to
+	 *	all zeros. Return `1' in that case.
+	 */
+	memset(zero, 0, sizeof(zero));
+	if (memcmp(authreq->vector, zero, AUTH_VECTOR_LEN) == 0)
+		return 1;
+
+	/*
+	 *	Zero out the auth_vector in the received packet.
+	 *	Then append the shared secret to the received packet,
+	 *	and calculate the MD5 sum. This must be the same
+	 *	as the original MD5 sum (authreq->vector).
+	 */
+	memset(recvbuf + 4, 0, AUTH_VECTOR_LEN);
+	memcpy(recvbuf + len, cl->secret, secretlen);
+	md5_calc(digest, recvbuf, len + secretlen);
+
+	/*
+	 *	Return 0 if OK, 2 if not OK.
+	 */
+	return memcmp(digest, authreq->vector, AUTH_VECTOR_LEN) ? 2 : 0;
 }
 
 
+/*
+ *	rad_accounting: call both the old and new style accounting functions.
+ */
+int rad_accounting(AUTH_REQ *authreq, int activefd, char *recvbuf, int len)
+{
+	int reply = 0;
+	int auth;
+	char pw_digest[16];
+
+	/*
+	 *	See if we know this client, then check the
+	 *	request authenticator.
+	 */
+	auth = calc_acctdigest(pw_digest, authreq, recvbuf, len);
+	if (auth < 0) {
+		pairfree(authreq->request);
+		memset(authreq, 0, sizeof(AUTH_REQ));
+		free(authreq);
+		return -1;
+	}
+
+	if (log_stripped_names) {
+		/*
+		 *	rad_accounting_new strips authreq for us. If
+		 *	we run it first, the stripped info will also
+		 *	get into the "details" file.
+		 */
+		if (rad_accounting_new(authreq, activefd, 1) == 0)
+			reply = 1;
+		if (rad_accounting_orig(authreq, activefd, auth) == 0)
+			reply = 1;
+	} else {
+		/*
+		 *	First log into the details file, before the
+		 *	username gets stripped by rad_accounting_new.
+		 */
+		if (rad_accounting_orig(authreq, activefd, auth) == 0)
+			reply = 1;
+		if (rad_accounting_new(authreq, activefd, 1) == 0)
+			reply = 1;
+	}
+
+	if (reply) {
+		/*
+		 *	Now send back an ACK to the NAS.
+		 */
+		send_acct_reply(authreq, (VALUE_PAIR *)NULL, 
+				(char *)NULL,activefd);
+	}
+
+	pairfree(authreq->request);
+	memset(authreq, 0, sizeof(AUTH_REQ));
+	free(authreq);
+
+	return reply ? 0 : -1;
+}
+
+
+#if 0 /* UNRELIABLE */
 /*
  *	See if a user is already logged in.
  *
- *	FIXME:	check twice. If on the first pass the user exceeds his
- *		max. number of logins, do a second pass and validate all
- *		logins by querying the terminal server (using eg. telnet).
- *
  *	For now we implement another algorithm. If the total no. of logins
  *	exceeds the maximum, we re-read the utmp file. We keep on trying
  *	for 30 seconds. This does not solve the "lost accounting records"
@@ -700,6 +829,196 @@ int rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul)
 	}
 	fclose(fp);
 
-	return -1;
+	return 1;
 }
 
+#else /* RELIABLE */
+
+/*
+ *	Timeout handler (10 secs)
+ */
+static int got_alrm;
+static void alrm_handler()
+{
+	got_alrm = 1;
+}
+
+/*
+ *	Check one terminal server to see if a user is logged in.
+ */
+static int rad_check_ts(struct radutmp *ut)
+{
+	int	pid, st, e;
+	int	n;
+	NAS	*nas;
+	char	address[16];
+	char	port[8];
+	char	session_id[12];
+	char	*s;
+	void    (*handler)(int);
+
+	/*
+	 *	Find NAS type.
+	 */
+	if ((nas = nas_find(ntohl(ut->nas_address))) == NULL) {
+		log(L_ERR, "Accounting: unknown NAS");
+		return -1;
+	}
+
+	/*
+	 *	Fork.
+	 */
+	handler = signal(SIGCHLD, SIG_DFL);
+	if ((pid = fork()) < 0) {
+		log(L_ERR, "Accounting: fork: %s", strerror(errno));
+		signal(SIGCHLD, handler);
+		return -1;
+	}
+
+	if (pid > 0) {
+		/*
+		 *	Parent - Wait for checklogin to terminate.
+		 *	We timeout in 10 seconds.
+		 */
+		got_alrm = 0;
+		signal(SIGALRM, alrm_handler);
+		alarm(10);
+		while((e = wait(&st)) != pid)
+			if (e < 0 && (errno != EINTR || got_alrm))
+				break;
+		alarm(0);
+		signal(SIGCHLD, handler);
+		if (got_alrm) {
+			kill(pid, SIGTERM);
+			sleep(1);
+			kill(pid, SIGKILL);
+			log(L_ERR, "Check-TS: timeout waiting for checkrad");
+			return 2;
+		}
+		if (e < 0) {
+			log(L_ERR, "Check-TS: unknown error in waitpid()");
+			return 2;
+		}
+		return WEXITSTATUS(st);
+	}
+
+	/*
+	 *	Child - exec checklogin with the right parameters.
+	 */
+	for (n = 32; n >= 3; n--)
+		close(n);
+
+	ipaddr2str(address, ntohl(ut->nas_address));
+	sprintf(port, "%d", ut->nas_port);
+	sprintf(session_id, "%.8s", ut->session_id);
+
+	s = CHECKRAD2;
+	execl(CHECKRAD2, "checkrad", nas->nastype, address, port,
+		ut->login, session_id, NULL);
+	if (errno == ENOENT) {
+		s = CHECKRAD1;
+		execl(CHECKRAD1, "checklogin", nas->nastype, address, port,
+			ut->login, session_id, NULL);
+	}
+	log(L_ERR, "Check-TS: exec %s: %s", s, strerror(errno));
+
+	/*
+	 *	Exit - 2 means "some error occured".
+	 */
+	exit(2);
+}
+
+/*
+ *	See if a user is already logged in.
+ *
+ *	Check twice. If on the first pass the user exceeds his
+ *	max. number of logins, do a second pass and validate all
+ *	logins by querying the terminal server (using eg. telnet).
+ *
+ *	Returns: 0 == OK, 1 == double logins, 2 == multilink attempt
+ */
+int rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul)
+{
+	FILE		*wfp;
+	int		fd;
+	int		count;
+	struct radutmp	u, empty;
+	struct utmp	wt;
+	VALUE_PAIR	*fra;
+	int		mpp = 1;
+	UINT4		ipno = 0;
+
+	if ((fd = open(RADUTMP, O_CREAT|O_RDWR, 0644)) < 0)
+		return 0;
+
+	/*
+	 *	We don't lock in the first pass.
+	 */
+	count = 0;
+	while(read(fd, &u, sizeof(u)) == sizeof(u))
+		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
+		    && u.type == P_LOGIN)
+			count++;
+
+	if (count < maxsimul) {
+		close(fd);
+		return 0;
+	}
+	lseek(fd, 0L, SEEK_SET);
+
+	/*
+	 *	Setup some stuff, like for MPP detection.
+	 */
+	if ((fra = pairfind(request, PW_FRAMED_IP_ADDRESS)) != NULL)
+		ipno = htonl(fra->lvalue);
+	memset(&empty, 0, sizeof(empty));
+
+	/*
+	 *	lockf() the file while reading/writing.
+	 */
+#if defined(F_LOCK) && !defined(BSD)
+		(void)lockf(fd, F_LOCK, LOCK_LEN);
+#else
+		(void)flock(fd, LOCK_EX);
+#endif
+
+	/*
+	 *	Allright, there are too many concurrent logins.
+	 *	Check all registered logins by querying the
+	 *	terminal server directly.
+	 */
+	count = 0;
+	while (read(fd, &u, sizeof(u)) == sizeof(u)) {
+		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
+		    && u.type == P_LOGIN) {
+			if (rad_check_ts(&u) == 1) {
+				count++;
+				/*
+				 *	Does it look like a MPP attempt?
+				 */
+				if (strchr("SCPA", u.proto) &&
+				    ipno && u.framed_address == ipno)
+					mpp = 2;
+			}
+			else {
+				/*
+				 *	False record - zap it.
+				 */
+
+				lseek(fd, -sizeof(u), SEEK_CUR);
+				write(fd, &empty, sizeof(empty));
+
+				if ((wfp = fopen(RADWTMP, "a")) != NULL) {
+					make_wtmp(&u, &wt, PW_STATUS_STOP);
+					fwrite(&wt, sizeof(wt), 1, wfp);
+					fclose(wfp);
+				}
+			}
+		}
+	}
+	close(fd);
+
+	return (count < maxsimul) ? 0 : mpp;
+}
+#endif
+
