@@ -6,6 +6,7 @@
 
 #define RADIUS_DICTIONARY	"dictionary"
 #define RADIUS_CLIENTS		"clients"
+#define RADIUS_NASLIST		"naslist"
 #define RADIUS_USERS		"users"
 #define RADIUS_HOLD		"holdusers"
 #define RADIUS_LOG		"radius.log"
@@ -16,3 +17,7 @@
 #define RADWTMP			"/var/log/radwtmp"
 
 #define RADIUS_PID		"/var/run/radiusd.pid"
+
+#define CHECKRAD1		"/usr/sbin/checkrad"
+#define CHECKRAD2		"/usr/local/sbin/checkrad"
+
