@@ -28,7 +28,7 @@
 
 char files_sccsid[] =
 "@(#)users.c	1.12 Copyright 1992 Livingston Enterprises Inc\n"
-"@(#)files.c	2.10 Copyright 1997 Cistron Internet Services B.V.";
+"@(#)files.c	2.27 Copyright 1997 Cistron Internet Services B.V.";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -75,6 +75,7 @@ PAIR_LIST	*users;
 PAIR_LIST	*huntgroups;
 PAIR_LIST	*hints;
 CLIENT		*clients;
+NAS		*naslist;
 
 /*
  *	Returns a copy of a pair list.
@@ -109,6 +110,7 @@ static void pairmove(VALUE_PAIR **to, VALUE_PAIR **from)
 {
 	VALUE_PAIR *tail, *i, *next;
 	VALUE_PAIR *last = NULL;
+	int has_password = 0;
 
 	if (*to == NULL) {
 		*to = *from;
@@ -116,13 +118,27 @@ static void pairmove(VALUE_PAIR **to, VALUE_PAIR **from)
 		return;
 	}
 
-	for(tail = *to; tail->next; tail = tail->next)
-		;
+	tail = *to;
+	for(i = *to; i; i = i->next) {
+		if (i->attribute == PW_PASSWORD ||
+		    i->attribute == PW_CHAP_PASSWORD ||
+		    i->attribute == PW_CRYPT_PASSWORD)
+			has_password = 1;
+		tail = i;
+	}
 
 	for(i = *from; i; i = next) {
 		next = i->next;
+		if (has_password &&
+		    (i->attribute == PW_PASSWORD ||
+		     i->attribute == PW_CHAP_PASSWORD ||
+		     i->attribute == PW_CRYPT_PASSWORD)) {
+			last = i;
+			continue;
+		}
 		if (i->attribute == PW_FALL_THROUGH ||
-		    pairfind(*to, i->attribute) != 0) {
+		    (i->attribute != PW_HINT &&
+		     pairfind(*to, i->attribute) != 0)) {
 			last = i;
 			continue;
 		}
@@ -136,6 +152,38 @@ static void pairmove(VALUE_PAIR **to, VALUE_PAIR **from)
 	}
 }
 
+/*
+ *	Move one kind of attributes from one list to the other
+ */
+void pairmove2(VALUE_PAIR **to, VALUE_PAIR **from, int attr)
+{
+	VALUE_PAIR *tail, *i, *next;
+	VALUE_PAIR *last = NULL;
+
+	if (*to == NULL)
+		return;
+
+	tail = *to;
+	for(i = *to; i; i = i->next)
+		tail = i;
+
+	for(i = *from; i; i = next) {
+		next = i->next;
+
+		if (i->attribute != attr)
+			continue;
+
+		if (last)
+			last->next = next;
+		else
+			*from = next;
+		tail->next = i;
+		i->next = NULL;
+		tail = i;
+	}
+}
+
+
 
 /*
  *	Turn printable string into correct tm struct entries
@@ -160,6 +208,49 @@ static void user_gettime(char *valstr, struct tm *tm)
 }
 
 
+/*
+ *	Strip a username, based on Prefix/Suffix from the "users" file.
+ *	Not 100% safe, since we don't compare attributes.
+ */
+void presuf_setup(VALUE_PAIR *request_pairs)
+{
+	PAIR_LIST	*pl;
+	VALUE_PAIR	*presuf_pair;
+	VALUE_PAIR	*name_pair;
+	VALUE_PAIR	*tmp;
+	char		name[32];
+
+	if ((name_pair = pairfind(request_pairs, PW_USER_NAME)) == NULL)
+		return;
+
+	for (pl = users; pl; pl = pl->next) {
+		/*
+		 *	Matching name?
+		 */
+		if (strncmp(pl->name, "DEFAULT", 7) != 0 &&
+		    strcmp(pl->name, name_pair->strvalue) != 0)
+			continue;
+		/*
+		 *	Find Prefix / Suffix.
+		 */
+		if ((presuf_pair = pairfind(pl->check, PW_PREFIX)) == NULL &&
+		    (presuf_pair = pairfind(pl->check, PW_SUFFIX)) == NULL)
+			continue;
+		if (presufcmp(presuf_pair, name_pair->strvalue, name) != 0)
+			continue;
+		/*
+		 *	See if username must be stripped.
+		 */
+		if ((tmp = pairfind(pl->check, PW_STRIP_USERNAME)) != NULL &&
+		    tmp->lvalue == 0)
+			continue;
+		strcpy(name_pair->strvalue, name);
+		break;
+	}
+}
+
+
+
 /*
  *	Compare a portno with a range.
  */
@@ -207,6 +298,42 @@ static int groupcmp(VALUE_PAIR *check, VALUE_PAIR *request)
 	return (pwd->pw_gid == grp->gr_gid) ? 0 : -1;
 }
 
+/*
+ *	Compare prefix/suffix.
+ */
+int presufcmp(VALUE_PAIR *check, char *name, char *rest)
+{
+	int len, namelen;
+	int ret = -1;
+
+#if 0 /* DEBUG */
+	printf("Comparing %s and %s, check->attr is %d\n",
+		name, check->strvalue, check->attribute);
+#endif
+
+	len = strlen(check->strvalue);
+	switch (check->attribute) {
+		case PW_PREFIX:
+			ret = strncmp(name, check->strvalue, len);
+			if (ret == 0 && rest)
+				strcpy(rest, name + len);
+			break;
+		case PW_SUFFIX:
+			namelen = strlen(name);
+			if (namelen < len)
+				break;
+			ret = strcmp(name + namelen - len, check->strvalue);
+			if (ret == 0 && rest) {
+				strncpy(rest, name, namelen - len);
+				rest[namelen - len] = 0;
+			}
+			break;
+	}
+	return ret;
+}
+
+
+
 /*
  *	Compare two pair lists except for the password information.
  *	Return 0 on match.
@@ -217,7 +344,7 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 	VALUE_PAIR *auth_item;
 	int result = 0;
 
-	while (result == 0 && check_item != (VALUE_PAIR *)NULL) {
+	while (result == 0 && check_item != NULL) {
 		switch (check_item->attribute) {
 			/*
 			 *	Attributes we skip during comparison.
@@ -225,8 +352,11 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 			 */
 			case PW_EXPIRATION:
 			case PW_PASSWORD:
-			case PW_AUTH_TYPE:
+			case PW_CHAP_PASSWORD:
+			case PW_CRYPT_PASSWORD:
+			case PW_AUTHTYPE:
 			case PW_SIMULTANEOUS_USE:
+			case PW_STRIP_USERNAME:
 				check_item = check_item->next;
 				continue;
 		}
@@ -234,15 +364,33 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 		 *	See if this item is present in the request.
 		 */
 		auth_item = request;
-		while(auth_item != (VALUE_PAIR *)NULL) {
-			if (check_item->attribute == auth_item->attribute ||
-			    check_item->attribute == PW_HUNTGROUP_NAME ||
-			    (check_item->attribute == PW_GROUP_NAME &&
-			     auth_item->attribute  == PW_USER_NAME))
-				break;
-			auth_item = auth_item->next;
+		for (; auth_item != NULL; auth_item = auth_item->next) {
+			switch (check_item->attribute) {
+				case PW_PREFIX:
+				case PW_SUFFIX:
+				case PW_GROUP_NAME:
+				case PW_GROUP:
+					if (auth_item->attribute  !=
+					    PW_USER_NAME)
+						continue;
+				case PW_HUNTGROUP_NAME:
+					break;
+				case PW_HINT:
+					if (auth_item->attribute !=
+					    check_item->attribute)
+						continue;
+					if (strcmp(check_item->strvalue,
+					    auth_item->strvalue) != 0)
+						continue;
+					break;
+				default:
+					if (auth_item->attribute !=
+					    check_item->attribute)
+						continue;
+			}
+			break;
 		}
-		if (auth_item == (VALUE_PAIR *)NULL) {
+		if (auth_item == NULL) {
 			result = -1;
 			continue;
 		}
@@ -254,12 +402,20 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 		switch(check_item->type) {
 
 			case PW_TYPE_STRING:
+				if (check_item->attribute == PW_PREFIX ||
+				    check_item->attribute == PW_SUFFIX) {
+					if (presufcmp(check_item,
+					    auth_item->strvalue, NULL) != 0)
+						result = -1;
+				}
+				else
 				if (check_item->attribute == PW_NAS_PORT_ID) {
 					if (portcmp(check_item, auth_item) != 0)
 						result = -1;
 				}
 			 	else
-				if (check_item->attribute == PW_GROUP_NAME) {
+				if (check_item->attribute == PW_GROUP_NAME ||
+				    check_item->attribute == PW_GROUP) {
 					if (groupcmp(check_item, auth_item)!=0)
 						result = -1;
 				}
@@ -287,7 +443,8 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 				result = -1;
 				break;
 		}
-		check_item = check_item->next;
+		if (result == 0)
+			check_item = check_item->next;
 	}
 
 	return result;
@@ -314,7 +471,7 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 			 */
 			case PW_EXPIRATION:
 			case PW_PASSWORD:
-			case PW_AUTH_TYPE:
+			case PW_AUTHTYPE:
 			case PW_SIMULTANEOUS_USE:
 				check_item = check_item->next;
 				continue;
@@ -325,13 +482,16 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 		auth_item = request;
 		while(auth_item != (VALUE_PAIR *)NULL) {
 			if (check_item->attribute == auth_item->attribute ||
-			    (check_item->attribute == PW_GROUP_NAME &&
+			    ((check_item->attribute == PW_GROUP_NAME ||
+			      check_item->attribute == PW_GROUP) &&
 			     auth_item->attribute  == PW_USER_NAME))
 				break;
 			auth_item = auth_item->next;
 		}
-		if (auth_item == (VALUE_PAIR *)NULL)
+		if (auth_item == (VALUE_PAIR *)NULL) {
+			check_item = check_item->next;
 			continue;
+		}
 
 		/*
 		 *	OK it is present now compare them.
@@ -345,7 +505,8 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 						result = 0;
 				}
 			 	else
-				if (check_item->attribute == PW_GROUP_NAME) {
+				if (check_item->attribute == PW_GROUP_NAME ||
+				    check_item->attribute == PW_GROUP) {
 					if (groupcmp(check_item, auth_item)==0)
 						result = 0;
 				}
@@ -379,6 +540,51 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 
 }
 
+/*
+ *	Copy username from the beginning of the buffer and return
+ *	a pointer to the first non-space charachter after that.
+ *
+ *	We allow "" and \ to include spaces in the username.
+ */
+static char *getusername(char *buffer, char *entry)
+{
+	char *ptr;
+	char *to;
+	int spc_seen = 0;
+
+	ptr = buffer;
+	to = entry;
+
+	while (*ptr && !spc_seen) {
+		switch (*ptr) {
+			case '"':
+				ptr++;
+				while (*ptr && *ptr != '"')
+					*to++ = *ptr++;
+				if (*ptr) ptr++;
+				break;
+			case '\\':
+				ptr++;
+				*to++ = *ptr;
+				if (*ptr) ptr++;
+				break;
+			case ' ':
+			case '\t':
+			case '\n':
+				spc_seen = 1;
+				break;
+			default:
+				*to++ = *ptr++;
+				break;
+		}
+	}
+	*to = 0;
+
+	while (*ptr == ' ' || *ptr == '\t' || *ptr == '\n')
+		ptr++;
+
+	return ptr;
+}
 
 #define FIND_MODE_NAME	0
 #define FIND_MODE_REPLY	1
@@ -393,7 +599,7 @@ static PAIR_LIST *file_read(char *file, int complain)
 	int		mode = FIND_MODE_NAME;
 	char		entry[256];
 	char		buffer[256];
-	char		*ptr;
+	char		*ptr, *s;
 	VALUE_PAIR	*check_tmp;
 	VALUE_PAIR	*reply_tmp;
 	PAIR_LIST	*pl = NULL, *last = NULL, *t;
@@ -404,7 +610,7 @@ static PAIR_LIST *file_read(char *file, int complain)
 	 */
 	if ((fp = fopen(file, "r")) == NULL) {
 		if (!complain) return NULL;
-		log(L_CONS|L_ERR, "Couldn't open %s for reading", buffer);
+		log(L_CONS|L_ERR, "Couldn't open %s for reading", file);
 		return NULL;
 	}
 
@@ -417,17 +623,48 @@ static PAIR_LIST *file_read(char *file, int complain)
 parse_again:
 		if(mode == FIND_MODE_NAME) {
 			/*
-			 * Find the entry starting with the users name
+			 *	Find the entry starting with the users name
 			 */
 			if (isspace(buffer[0])) continue;
 
-			for(ptr = buffer; !isspace(*ptr) && *ptr; ptr++)
+			/*
+			 *	FIXME:	support `\ ' space in usernames.
+			 *		And "user name" ofcourse.
+			 */
+#if 1
+			ptr = getusername(buffer, entry);
+#else
+			for (ptr = buffer; !isspace(*ptr) && *ptr; ptr++)
 				;
 			strncpy(entry, buffer, ptr - buffer);
 			entry[ptr - buffer] = 0;
+#endif
 
 			/*
-			 * Parse the check values
+			 *	Include another file if we see
+			 *	$INCLUDE filename
+			 */
+			if (strcasecmp(entry, "$include") == 0) {
+				while(isspace(*ptr))
+					ptr++;
+				s = ptr;
+				while (!isspace(*ptr))
+					ptr++;
+				*ptr = 0;
+				if ((t = file_read(s, 1)) == NULL)
+					continue;
+				if (last)
+					last->next = t;
+				else
+					pl = t;
+				last = t;
+				while (last && last->next)
+					last = last->next;
+				continue;
+			}
+
+			/*
+			 *	Parse the check values
 			 */
 			check_tmp = NULL;
 			reply_tmp = NULL;
@@ -591,7 +828,6 @@ static int fallthrough(VALUE_PAIR *vp)
 int user_find(char *name, VALUE_PAIR *request_pairs,
 		VALUE_PAIR **check_pairs, VALUE_PAIR **reply_pairs)
 {
-	char		*ptr;
 	int		nas_port = 0;
 	VALUE_PAIR	*check_tmp;
 	VALUE_PAIR	*reply_tmp;
@@ -606,16 +842,8 @@ int user_find(char *name, VALUE_PAIR *request_pairs,
 	*reply_pairs = NULL;
 
 	/* 
-	 * Check for valid input, zero length names not permitted 
+	 *	Check for valid input, zero length names not permitted 
 	 */
-	ptr = name;
-	while (*ptr != '\0') {
-		if (*ptr == ' ' || *ptr == '\t') {
-			*ptr = '\0';
-		} else {
-			ptr++;
-		}
-	}
 	if (name[0] == 0) {
 		log(L_ERR, "zero length username not permitted\n");
 		return -1;
@@ -633,7 +861,7 @@ int user_find(char *name, VALUE_PAIR *request_pairs,
 #ifdef USE_DBM
 	if (use_dbm) {
 		/*
-		 *	FIXME: No Prefix / Suffix support.
+		 *	FIXME: No Prefix / Suffix support for DBM.
 		 */
 		sprintf(buffer, "%s/%s", radius_dir, RADIUS_USERS);
 #ifdef DBM
@@ -679,9 +907,7 @@ int user_find(char *name, VALUE_PAIR *request_pairs,
 #endif
 
 	for(pl = users; pl; pl = pl->next) {
-		/*
-		 *	FIXME: add Prefix/Suffix support.
-		 */
+
 		if (strcmp(name, pl->name) && strcmp(pl->name, "DEFAULT"))
 			continue;
 
@@ -930,11 +1156,43 @@ static int userparse(char *buffer, VALUE_PAIR **first_pair)
  *	Match a username with a wildcard expression.
  *	Is very limited for now.
  */
-static int matches(char *name, char *wild, char *matchpart)
+static int matches(char *name, PAIR_LIST *pl, char *matchpart)
 {
 	int len, wlen;
 	int ret = 0;
+	char *wild = pl->name;
+	VALUE_PAIR *tmp;
 
+	/*
+	 *	We now support both:
+	 *
+	 *		DEFAULT	Prefix = "P"
+	 *
+	 *	and
+	 *		P*
+	 */
+	if ((tmp = pairfind(pl->check, PW_PREFIX)) != NULL ||
+	    (tmp = pairfind(pl->check, PW_SUFFIX)) != NULL) {
+
+		if (strncmp(pl->name, "DEFAULT", 7) == 0 ||
+		    strcmp(pl->name, name) == 0)
+			return !presufcmp(tmp, name, matchpart);
+	}
+
+	/*
+	 *	Shortcut if there's no '*' in pl->name.
+	 */
+	if (strchr(pl->name, '*') == NULL &&
+	    (strncmp(pl->name, "DEFAULT", 7) == 0 ||
+	     strcmp(pl->name, name) == 0)) {
+		strcpy(matchpart, name);
+		return 1;
+	}
+
+	/*
+	 *	Normally, we should return 0 here, but we
+	 *	support the old * stuff.
+	 */
 	len = strlen(name);
 	wlen = strlen(wild);
 
@@ -967,13 +1225,16 @@ static int matches(char *name, char *wild, char *matchpart)
  */
 int hints_setup(VALUE_PAIR *request_pairs)
 {
-	char		newname[32];
+	char		newname[128];
 	char		*name;
-	char		*ptr;
 	VALUE_PAIR	*add;
 	VALUE_PAIR	*last;
 	VALUE_PAIR	*tmp;
 	PAIR_LIST	*i;
+	int		do_strip;
+#ifdef NT_DOMAIN_HACK
+	char		*ptr;
+#endif
 
 	if (hints == NULL)
 		return 0;
@@ -985,12 +1246,7 @@ int hints_setup(VALUE_PAIR *request_pairs)
 		log(L_ERR,"no user name");
 		return -1;
 	}
-	ptr = name = tmp->strvalue;
-	for(ptr = name; *ptr; ptr++)
-		if (*ptr == ' ' || *ptr == '\t') {
-			*ptr = 0;
-			break;
-		}
+	name = tmp->strvalue;
 
 	if (name[0] == 0)
 		/*
@@ -1006,13 +1262,10 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	 *	FIXME: should we handle this as a REALM ? (When we
 	 *	have implemented REALMS and proxy services, that is).
 	 */
-	if (tmp) {
-		char buffer[128];
-		if ((ptr = strchr(tmp->strvalue, '\\')) != NULL) {
-			strncpy(buffer, ptr + 1, sizeof(buffer));
-			buffer[sizeof(buffer) - 1] = 0;
-			strcpy(tmp->strvalue, buffer);
-		}
+	if ((ptr = strchr(name, '\\')) != NULL) {
+		strncpy(newname, ptr + 1, sizeof(newname));
+		newname[sizeof(newname) - 1] = 0;
+		strcpy(name, newname);
 	}
 #endif /* NT_DOMAIN_HACK */
 
@@ -1032,8 +1285,8 @@ int hints_setup(VALUE_PAIR *request_pairs)
 		}
 	}
 
-	for(i = hints; i; i = i->next) {
-		if (matches(name, i->name, newname))
+	for (i = hints; i; i = i->next) {
+		if (matches(name, i, newname))
 			break;
 	}
 
@@ -1041,11 +1294,22 @@ int hints_setup(VALUE_PAIR *request_pairs)
 
 	add = paircopy(i->reply);
 
+#if 0 /* DEBUG */
+	printf("In hints_setup, newname is %s\n", newname);
+#endif
+
 	/*
 	 *	See if we need to adjust the name.
 	 */
-	if ((tmp = pairfind(i->reply, PW_USER_NAME_IS_STAR)) != NULL
-	    && tmp->lvalue) {
+	do_strip = 1;
+	if ((tmp = pairfind(i->reply, PW_STRIP_USERNAME)) != NULL
+	     && tmp->lvalue == 0)
+		do_strip = 0;
+	if ((tmp = pairfind(i->check, PW_STRIP_USERNAME)) != NULL
+	     && tmp->lvalue == 0)
+		do_strip = 0;
+
+	if (do_strip) {
 		tmp = pairfind(request_pairs, PW_USER_NAME);
 		if (tmp)
 			strcpy(tmp->strvalue, newname);
@@ -1053,9 +1317,9 @@ int hints_setup(VALUE_PAIR *request_pairs)
 
 	/*
 	 *	Now add all attributes to the request list,
-	 *	except the PW_USER_NAME_IS_STAR one.
+	 *	except the PW_STRIP_USERNAME one.
 	 */
-	pairdelete(&add, PW_USER_NAME_IS_STAR);
+	pairdelete(&add, PW_STRIP_USERNAME);
 	for(last = request_pairs; last && last->next; last = last->next)
 		;
 	if (last) last->next = add;
@@ -1063,7 +1327,6 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	return 0;
 }
 
-
 /*
  *	See if the huntgroup matches.
  */
@@ -1248,6 +1511,124 @@ char *client_name(UINT4 ipaddr)
 	return ip_hostname(ipaddr);
 }
 
+/*
+ *	Free a NAS list.
+ */
+static void nas_free(NAS *cl)
+{
+	NAS *next;
+
+	while(cl) {
+		next = cl->next;
+		free(cl);
+		cl = next;
+	}
+}
+
+/*
+ *	Read the nas file.
+ */
+int read_naslist_file(char *file)
+{
+	FILE	*fp;
+	char	buffer[256];
+	char	hostnm[128];
+	char	shortnm[32];
+	char	nastype[32];
+	int	lineno = 0;
+	NAS	*c;
+
+	nas_free(naslist);
+
+	if ((fp = fopen(file, "r")) == NULL) {
+		log(L_CONS|L_ERR, "cannot open %s", file);
+		return -1;
+	}
+	while(fgets(buffer, 256, fp) != NULL) {
+		lineno++;
+		if (buffer[0] == '#' || buffer[0] == '\n')
+			continue;
+		nastype[0] = 0;
+		if (sscanf(buffer, "%s%s%s", hostnm, shortnm, nastype) < 2) {
+			log(L_ERR, "%s[%d]: syntax error", file, lineno);
+			continue;
+		}
+		if ((c = malloc(sizeof(NAS))) == NULL) {
+			log(L_CONS|L_ERR, "%s[%d]: out of memory",
+				file, lineno);
+			return -1;
+		}
+
+		c->ipaddr = get_ipaddr(hostnm);
+		strcpy(c->nastype, nastype);
+		strcpy(c->shortname, shortnm);
+		strcpy(c->longname, ip_hostname(c->ipaddr));
+
+		c->next = naslist;
+		naslist = c;
+	}
+	fclose(fp);
+
+	return 0;
+}
+
+
+/*
+ *	Find a nas in the NAS list.
+ */
+NAS *nas_find(UINT4 ipaddr)
+{
+	NAS *cl;
+
+	for(cl = naslist; cl; cl = cl->next)
+		if (ipaddr == cl->ipaddr)
+			break;
+
+	return cl;
+}
+
+
+/*
+ *	Find the name of a nas (prefer short name).
+ */
+char *nas_name(UINT4 ipaddr)
+{
+	NAS *cl;
+
+	if ((cl = nas_find(ipaddr)) != NULL) {
+		if (cl->shortname[0])
+			return cl->shortname;
+		else
+			return cl->longname;
+	}
+	return ip_hostname(ipaddr);
+}
+
+/*
+ *	Find the name of a nas (prefer short name) based on the request.
+ *	FIXME: the implementation in acct.c:rad_accounting_orig() is better.
+ */
+char *nas_name2(AUTH_REQ *authreq)
+{
+	UINT4	ipaddr;
+	NAS	*cl;
+	VALUE_PAIR	*pair;
+
+	if ((pair = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL)
+		ipaddr = pair->lvalue;
+	else
+		ipaddr = authreq->ipaddr;
+
+	if ((cl = nas_find(ipaddr)) != NULL) {
+		if (cl->shortname[0])
+			return cl->shortname;
+		else
+			return cl->longname;
+	}
+	return ip_hostname(ipaddr);
+}
+
+
 
 #if USE_DBM
 /*
@@ -1292,49 +1673,10 @@ int read_config_files()
 	sprintf(buffer, "%s/%s", radius_dir, RADIUS_CLIENTS);
 	if (read_clients_file(buffer) < 0)
 		return -1;
+	sprintf(buffer, "%s/%s", radius_dir, RADIUS_NASLIST);
+	if (read_naslist_file(buffer) < 0)
+		return -1;
 
 	return 0;
 }
 
-
-#ifdef CISTRON_COMPAT
-/*
- *	The first Cistron Terminal Servers sent the following data
- *	upon a successful connect:
- *
- *	Your IP address: 192.168.1.42
- *	 Our IP address: 192.168.1.1
- *
- *	Entering PPP mode
- *
- *	This function makes sure the Portmasters do the same.
- */
-char *cistron_compat(VALUE_PAIR *request, VALUE_PAIR *reply)
-{
-	VALUE_PAIR	*framed;
-	VALUE_PAIR	*nas;
-	VALUE_PAIR	*proto;
-	static char	buf[512];
-	char		buf1[16];
-	char		buf2[16];
-
-	if ((framed = pairfind(reply, PW_FRAMED_IP_ADDRESS)) == NULL ||
-	    (nas    = pairfind(request, PW_NAS_IP_ADDRESS)) == NULL ||
-	    (proto  = pairfind(reply, PW_FRAMED_PROTOCOL)) == NULL)
-		return 0;
-
-	ipaddr2str(buf1, framed->lvalue);
-	ipaddr2str(buf2, nas->lvalue);
-
-	sprintf(buf, "\r\n"
-		"Your IP address: %s\r\n"
-		" Our IP address: %s\r\n"
-		"\r\n"
-		"Entering %s mode.\r\n",
-		buf1, buf2, proto->lvalue == PW_PPP ? "PPP" : "SLIP");
-		
-
-	return buf;
-}
-#endif
-
