@@ -90,6 +90,7 @@ typedef struct pw_auth_hdr {
 #define PW_FRAMED_ROUTE			22
 #define PW_FRAMED_IPXNET		23
 #define PW_STATE			24
+#define PW_PROXY_STATE			33
 
 #define PW_ACCT_STATUS_TYPE		40
 #define PW_ACCT_DELAY_TIME		41
@@ -103,16 +104,24 @@ typedef struct pw_auth_hdr {
 
 #define PW_NAS_PORT_TYPE		61
 #define PW_PORT_LIMIT			62
+#define PW_CONNECT_INFO			77
 
 #define PW_HUNTGROUP_NAME		221
-#define PW_AUTH_TYPE			1000
+#define PW_AUTHTYPE			1000
+#define PW_PREFIX			1003
+#define PW_SUFFIX			1004
+#define PW_GROUP			1005
+#define PW_CRYPT_PASSWORD		1006
+#define PW_CONNECT_RATE			1007
 #define PW_USER_CATEGORY		1029
 #define PW_GROUP_NAME			1030
 #define PW_SIMULTANEOUS_USE		1034
-#define PW_USER_NAME_IS_STAR		1035
+#define PW_STRIP_USERNAME		1035
 #define PW_FALL_THROUGH			1036
 #define PW_ADD_PORT_TO_IP_ADDRESS	1037
 #define PW_EXEC_PROGRAM			1038
+#define PW_EXEC_PROGRAM_WAIT		1039
+#define PW_HINT				1040
 
 /*
  *	INTEGER TRANSLATIONS
@@ -150,10 +159,20 @@ typedef struct pw_auth_hdr {
 
 /*	AUTHENTICATION LEVEL	*/
 
-#define PW_AUTH_LOCAL			0
-#define PW_AUTH_SYSTEM			1
-#define PW_AUTH_SECURID			2
-#define PW_AUTH_NONE			254
+#define PW_AUTHTYPE_LOCAL		0
+#define PW_AUTHTYPE_SYSTEM		1
+#define PW_AUTHTYPE_SECURID		2
+#define PW_AUTHTYPE_CRYPT		3
+#define PW_AUTHTYPE_REJECT		4
+#define PW_AUTHTYPE_PAM			253
+#define PW_AUTHTYPE_NONE		254
+
+/*	PORT TYPES		*/
+#define PW_NAS_PORT_ASYNC		0
+#define PW_NAS_PORT_SYNC		1
+#define PW_NAS_PORT_ISDN		2
+#define PW_NAS_PORT_ISDN_V120		3
+#define PW_NAS_PORT_ISDN_V110		4
 
 /*	STATUS TYPES	*/
 
