@@ -31,27 +31,6 @@
 #define P_IDLE		0
 #define P_LOGIN		1
 
-#if 0
-
-/*
- *	Old structures - not used anymore.
- */
-#define RUT_HOSTSIZE 32
-#define RUT_NAMESIZE 16
-#define RUT_LINESIZE 12
-#define RUT_SERVSIZE 32
-
-struct radutmp {
-  char ut_name[RUT_NAMESIZE];  /* Loginname */
-  char ut_host[RUT_HOSTSIZE];  /* Hostname [dynamic IP address] */
-  char ut_line[RUT_LINESIZE];  /* port id */
-  char ut_serv[RUT_SERVSIZE];  /* Name of terminal server. */
-  char ut_id[4];               /* Ppp, Slip, Terminal */
-  time_t ut_time;              /* Start of login. */
-};
-
-#else
-
 #define RUT_NAMESIZE 32
 
 struct radutmp {
@@ -64,9 +43,9 @@ struct radutmp {
   time_t time;			/* Time entry was last updated. */
   time_t delay;			/* Delay time of request */
   int type;			/* Type of entry (login/logout) */
-  char reserved[32];		/* 8 ints reserved */
+  char porttype;		/* Porttype (I=ISDN A=Async T=Async-ISDN */
+  char res1,res2,res3;		/* Fills up to one int */
+  char reserved[28];		/* 7 ints reserved */
 };
 
-#endif /* RADUTMP_VER */
-
 #endif /* _RADUTMP_H */
