@@ -37,7 +37,7 @@ char version_sccsid[] =
  *  would like a range of versions allocated for your use.
  */
 
-#define		VERSION		"cistron-1.5.2 19-Feb-1997"
+#define		VERSION		"cistron-1.5.4 21-Oct-1997"
 
 /*
  *	Display the revision number for this program
@@ -46,6 +46,7 @@ void version(void)
 {
 
 	fprintf(stderr, "%s: RADIUS version %s\n", progname, VERSION);
+	fprintf(stderr, "Compilation flags: ");
 
 	/* here are all the conditional feature flags */
 #if defined(DBM)
@@ -57,9 +58,6 @@ void version(void)
 #if defined(NOSHADOW)
 	fprintf(stderr," NOSHADOW");
 #endif
-#if defined(CISTRON_COMPAT)
-	fprintf(stderr," CISTRON_COMPAT");
-#endif
 #if defined(NT_DOMAIN_HACK)
 	fprintf(stderr," NT_DOMAIN_HACK");
 #endif
